SystemOrganization addCategory: #'SUnit-Kernel'!
SystemOrganization addCategory: #'SUnit-Extensions'!
SystemOrganization addCategory: #'SUnit-Tests'!
SystemOrganization addCategory: #SUnit!

Object subclass: #TestCase
	instanceVariableNames: 'testSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SUnit-Kernel'!

!TestCase commentStamp: 'kph 12/15/2006 00:45' prior: 0!
A TestCase is a Command representing the future running of a test case. Create one with the class method #selector: aSymbol, passing the name of the method to be run when the test case runs.

When you discover a new fixture, subclass TestCase, declare instance variables for the objects in the fixture, override #setUp to initialize the variables, and possibly override# tearDown to deallocate any external resources allocated in #setUp.

When you are writing a test case method, send #assert: aBoolean when you want to check for an expected value. For example, you might say "self assert: socket isOpen" to test whether or not a socket is open at a point in a test.

Modification Notes: KPH 26/10/06
======================================
Removed the magic that a TestCase subclass with no test selectors automatically inherits testSelectors. This can and should be explicitly set by overriding #shouldInheritSelectors to true so as to make the behaviour explicit.

Simplification/refactoring:

    All hard wired references to 'test*' have been removed from the suiteBuild protocol, indeed the suiteBuild protocol has been greatly simplified/removed. The same capability is implemented with 6 less methods on the class side of TestCase

More flexible suite building API:

Asking an abstract class for #suite: will build the entire suite for all of its concrete subclasses.
Asking a concrete class for #suite: will just return the suite for that one class.

The suite building api now matches against method selector and category ("selector@category").

Multiple matches may be specified.
Exclusions may be defined.

Suites are defined as class methods for a testcase class.
example:
	myTestSuite
		"all 'test' methods, but not those in categories 'long tests', or 'broken' 
		^ 'test*|^*@long tests|^*@broken'

Suites are obtained via either a) a selector which references the defined suite or b) an explicit string.

	TestCase suite: #myTestSuite.		
	TestCase suite: 'test*|longtest*'

Suites may be defined as an array combining other suites
example:
	myTestSuiteWithLongTests
		^ #( #myTestSuite '*@long tests' ) 

Published Suites API

#publishedSuites provides a list of test suites published by each TestCase class that can be picked in the TestRunner. This provides a natural UI for selecting from a range of testing scenarios. e.g. tests for different products, releases, platforms, performance monitoring tests, long tests, tests needing additional services (db)

    i.e TestCase publishes #standardTests, whereas LongTestCase publishes #longTests, but not #standardTests.

LongTestCase Simplified

Any class can now mark some tests as being long and mark them to be published accordingly. i.e. #longTests ^self suite: 'longtest*'. Alternatively any class can mark all of its tests as long i.e. #longTests ^self suite: 'test*|longtest*', and not publish #standardTests.

With this new publishing suites API the functionality of LongTestCase is no longer needed. It could be removed but is left for backwards compatibility.

Published Suites API enables categories of tests

    This allows testMethods to be individually categorised and to belong to multiple categories.

    examples of categories might be

       1. tests that have not been written yet #wipTests
       2. tests specific to v39
       3. tests specific to v310 etc...
       4. test examples including demonstrations of failures and errors #exampleFailures

Additional asserts

added #assert:equals and #assert:includes  

 !

TestCase subclass: #ClassClonerTestResourceTest
	instanceVariableNames: 'originalClass clonedClass'
	classVariableNames: 'ClassVar'
	poolDictionaries: ''
	category: 'SUnit-Tests'!

!ClassClonerTestResourceTest class methodsFor: 'as yet unclassified' stamp: 'kph 12/20/2006 20:32'!
changeClassVar

ClassVar := 'changed'.! !

!ClassClonerTestResourceTest class methodsFor: 'as yet unclassified' stamp: 'kph 12/20/2006 20:32'!
classVar

^ ClassVar! !

!ClassClonerTestResourceTest class methodsFor: 'as yet unclassified' stamp: 'kph 12/20/2006 20:32'!
initialize

 ClassVar := 'initialized'.! !

!ClassClonerTestResourceTest class methodsFor: 'as yet unclassified' stamp: 'kph 12/2/2006 05:11'!
resources

^ Set with: ClassClonerTestResource! !

!ClassClonerTestResourceTest methodsFor: 'as yet unclassified' stamp: 'kph 12/20/2006 20:33'!
setUp

originalClass := self class initialize.
clonedClass := ClassClonerTestResource current cloneOf: originalClass.
clonedClass initialize.! !

!ClassClonerTestResourceTest methodsFor: 'as yet unclassified' stamp: 'kph 12/20/2006 20:05'!
testChangeClone

clonedClass changeClassVar.

self assert: (clonedClass classVar = 'changed').
self assert: (originalClass classVar = 'initialized')! !

!ClassClonerTestResourceTest methodsFor: 'as yet unclassified' stamp: 'kph 12/20/2006 20:04'!
testInitialState

self assert: (clonedClass classVar = 'initialized').
self assert: (originalClass classVar = 'initialized').
self deny: (clonedClass classVar == originalClass classVar). 
self deny: ((clonedClass classPool at: #ClassVar) == (originalClass classPool at: #ClassVar)). ! !

TestCase subclass: #LongTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SUnit-Extensions'!

!LongTestCase commentStamp: 'kph 12/14/2006 03:43' prior: 0!
The ability to define categories of tests such as LongTestCase has been rolled into TestCase.
This class is present for compatibility only, but is now obsolete.
!

!LongTestCase class methodsFor: 'as yet unclassified' stamp: 'kph 12/14/2006 03:49'!
longTests

"all tests are long"

^ 'test*|longtest*'! !

!LongTestCase class methodsFor: 'as yet unclassified' stamp: 'kph 12/14/2006 22:19'!
myTests

"all tests are long"

^ 'test*|longtest*'! !

!LongTestCase class methodsFor: 'as yet unclassified' stamp: 'kph 12/14/2006 22:18'!
publishedSuites
	^ #(#longTests #myTests)! !

!LongTestCase class methodsFor: 'as yet unclassified' stamp: 'kph 12/14/2006 03:49'!
standardTests

"all tests are long"

^ nil! !

TestCase subclass: #ResumableTestFailureTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SUnit-Tests'!

!ResumableTestFailureTestCase methodsFor: 'Not categorized' stamp: ''!
errorTest
	1 zork
			! !

!ResumableTestFailureTestCase methodsFor: 'Not categorized' stamp: 'md 2/22/2006 14:17'!
failureLog
	^Transcript
			! !

!ResumableTestFailureTestCase methodsFor: 'Not categorized' stamp: ''!
failureTest
	self
		assert: false description: 'You should see me' resumable: true; 
		assert: false description: 'You should see me too' resumable: true; 
		assert: false description: 'You should see me last' resumable: false; 
		assert: false description: 'You should not see me' resumable: true
			! !

!ResumableTestFailureTestCase methodsFor: 'Not categorized' stamp: ''!
isLogging
	^false
			! !

!ResumableTestFailureTestCase methodsFor: 'Not categorized' stamp: ''!
okTest
	self assert: true
			! !

!ResumableTestFailureTestCase methodsFor: 'Not categorized' stamp: ''!
regularTestFailureTest
	self assert: false description: 'You should see me'
			! !

!ResumableTestFailureTestCase methodsFor: 'Not categorized' stamp: ''!
resumableTestFailureTest
	self
		assert: false description: 'You should see me' resumable: true; 
		assert: false description: 'You should see me too' resumable: true; 
		assert: false description: 'You should see me last' resumable: false; 
		assert: false description: 'You should not see me' resumable: true
			! !

!ResumableTestFailureTestCase methodsFor: 'Not categorized' stamp: ''!
testResumable
	| result suite |
	suite := TestSuite new.
	suite addTest: (self class selector: #errorTest).
	suite addTest: (self class selector: #regularTestFailureTest).
	suite addTest: (self class selector: #resumableTestFailureTest).
	suite addTest: (self class selector: #okTest).
	result := suite run.
	self assert: result failures size = 2;
		assert: result errors size = 1
			! !

TestCase subclass: #SUnitExtensionsTest
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SUnit-Tests'!

!SUnitExtensionsTest methodsFor: 'real tests' stamp: 'mx 3/20/2006 23:47'!
assertionFailedInRaiseWithExceptionDoTest

	self 
		should: [ Error signal ]
		raise: Error
		withExceptionDo: [ :anException | self assert: false ]! !

!SUnitExtensionsTest methodsFor: 'real tests' stamp: 'mx 3/20/2006 23:47'!
differentExceptionInShouldRaiseWithExceptionDoTest

	[ self 
		should: [ Error signal ]
		raise: Halt
		withExceptionDo: [ :anException | self assert: false description: 'should:raise:withExceptionDo: handled an exception that should not handle'] ]
	on: Error
	do: [ :anException | anException return: nil ]! !

!SUnitExtensionsTest methodsFor: 'real tests' stamp: 'mx 3/20/2006 23:47'!
errorInRaiseWithExceptionDoTest

	self 
		should: [ Error  signal ]
		raise: Error
		withExceptionDo: [ :anException | Error signal: 'A forced error' ]! !

!SUnitExtensionsTest methodsFor: 'test support' stamp: ''!
failureLog
	^self stream! !

!SUnitExtensionsTest methodsFor: 'real tests' stamp: 'md 8/2/2006 11:04'!
invalidShouldNotTakeMoreThan

	self should: [(Delay forMilliseconds: 100) wait] notTakeMoreThan: 50 milliSeconds.! !

!SUnitExtensionsTest methodsFor: 'real tests' stamp: ''!
invalidShouldNotTakeMoreThanMilliseconds

	self should: [(Delay forMilliseconds: 100) wait] notTakeMoreThanMilliseconds: 50! !

!SUnitExtensionsTest methodsFor: 'testing' stamp: ''!
isLogging
	^true! !

!SUnitExtensionsTest methodsFor: 'real tests' stamp: 'mx 3/20/2006 23:47'!
noExceptionInShouldRaiseWithExceptionDoTest

	self 
		should: [  ]
		raise: Error
		withExceptionDo: [ :anException | Error signal: 'Should not get here' ]! !

!SUnitExtensionsTest methodsFor: 'real tests' stamp: ''!
shouldFixTest

	self shouldFix: [ Error signal: 'any kind of error' ]
! !

!SUnitExtensionsTest methodsFor: 'real tests' stamp: 'mx 3/20/2006 23:47'!
shouldRaiseWithExceptionDoTest

	self 
		should: [ Error signal: '1' ]
		raise: Error
		withExceptionDo: [ :anException | self assert: anException messageText = '1' ]! !

!SUnitExtensionsTest methodsFor: 'real tests' stamp: 'mx 3/20/2006 23:47'!
shouldRaiseWithSignalDoTest

	self 
		should: [ Error signal: '1' ]
		raise: Error
		withExceptionDo: [ :anException | self assert: anException messageText = '1' ]! !

!SUnitExtensionsTest methodsFor: 'accessing' stamp: ''!
stream
	stream isNil ifTrue: [stream := WriteStream on: String new].
	^stream! !

!SUnitExtensionsTest methodsFor: 'test' stamp: 'mx 3/20/2006 23:39'!
testAssertionFailedInRaiseWithExceptionDo

	| testCase testResult  |
	
	testCase := self class selector: #assertionFailedInRaiseWithExceptionDoTest.
	testResult := testCase run.
	
	self assert: (testResult failures includes: testCase).
	self assert: testResult failures size=1.
	self assert: testResult passed isEmpty.
	self assert: testResult errors isEmpty.
	
	! !

!SUnitExtensionsTest methodsFor: 'test' stamp: 'mx 3/20/2006 21:15'!
testAutoAssertFalse
	| booleanCondition |
	self assert: self isLogging.
	self should: [ self assert: 1 = 2 description: 'self assert: 1 = 2' ] raise: TestResult failure.
	booleanCondition := (self stream contents subStrings: {Character cr}) last = 'self assert: 1 = 2'.
	self assert: booleanCondition! !

!SUnitExtensionsTest methodsFor: 'test' stamp: ''!
testAutoAssertTrue
	self assert: 1 = 1.
	self assert: true! !

!SUnitExtensionsTest methodsFor: 'test' stamp: 'mx 3/20/2006 21:16'!
testAutoDenyFalse
	| booleanCondition |
	self assert: self isLogging.
	self should: [ self deny: 1 = 1 description: 'self deny: 1 = 1'.] raise: TestResult failure.
	booleanCondition := (self stream contents subStrings:  {Character cr}) last = 'self deny: 1 = 1'.
	self assert: booleanCondition! !

!SUnitExtensionsTest methodsFor: 'test' stamp: ''!
testAutoDenyTrue
	self deny: 1 = 2.
	self deny: false! !

!SUnitExtensionsTest methodsFor: 'test' stamp: 'mx 3/20/2006 23:40'!
testDifferentExceptionInShouldRaiseWithExceptionDo

	| testCase testResult  |
	
	testCase := self class selector: #differentExceptionInShouldRaiseWithExceptionDoTest.
	testResult := testCase run.
	
	self assert: (testResult passed includes: testCase).
	self assert: testResult errors isEmpty.
	self assert: testResult failures isEmpty.
	self assert: testResult passed size=1! !

!SUnitExtensionsTest methodsFor: 'test' stamp: 'mx 3/20/2006 23:40'!
testErrorInRaiseWithExceptionDo

	| testCase testResult  |
	
	testCase := self class selector: #errorInRaiseWithExceptionDoTest.
	testResult := testCase run.
		
	self assert: (testResult errors includes: testCase).
	self assert: testResult errors size=1.
	self assert: testResult failures isEmpty.
	self assert: testResult passed isEmpty.
	
	! !

!SUnitExtensionsTest methodsFor: 'tests' stamp: 'md 4/2/2006 14:02'!
testExceptionWithMatchingString
	self should: [ Object obsolete ] raise: Error whoseDescriptionIncludes: 'NOT obsolete' description: 'tested obsoleting Object'! !

!SUnitExtensionsTest methodsFor: 'tests' stamp: 'md 4/2/2006 14:02'!
testExceptionWithoutMatchingString
	self should: [ Object obsolete ] raise: Error whoseDescriptionDoesNotInclude: 'Zero' description: 'tested obsoleting Object'! !

!SUnitExtensionsTest methodsFor: 'test' stamp: 'md 8/2/2006 11:06'!
testInvalidShouldNotTakeMoreThan

	| testCase testResult |

	testCase := self class selector: #invalidShouldNotTakeMoreThan.
	testResult := testCase run.

	self assert: testResult passed isEmpty.
	self assert: testResult failures size = 1.
	self assert: (testResult failures includes: testCase).
	self assert: testResult errors isEmpty

! !

!SUnitExtensionsTest methodsFor: 'test' stamp: ''!
testInvalidShouldNotTakeMoreThanMilliseconds

	| testCase testResult |

	testCase := self class selector: #invalidShouldNotTakeMoreThanMilliseconds.
	testResult := testCase run.

	self assert: testResult passed isEmpty.
	self assert: testResult failures size = 1.
	self assert: (testResult failures includes: testCase).
	self assert: testResult errors isEmpty

! !

!SUnitExtensionsTest methodsFor: 'test' stamp: 'mx 3/20/2006 23:40'!
testNoExceptionInShouldRaiseWithExceptionDo

	| testCase testResult  |
	
	testCase := self class selector: #noExceptionInShouldRaiseWithExceptionDoTest.
	testResult := testCase run.
	
	self assert: (testResult failures includes: testCase).
	self assert: testResult failures size=1.
	self assert: testResult passed isEmpty.
	self assert: testResult errors isEmpty.
	
	! !

!SUnitExtensionsTest methodsFor: 'tests' stamp: 'md 4/2/2006 14:02'!
testNoExceptionWithMatchingString
	self shouldnt: [ Object obsolete ] raise: Error whoseDescriptionIncludes: 'Zero' description: 'tested obsoleting Object'! !

!SUnitExtensionsTest methodsFor: 'tests' stamp: 'md 4/2/2006 14:02'!
testNoExceptionWithNoMatchingString
	self shouldnt: [ Object obsolete ] raise: Error whoseDescriptionDoesNotInclude: 'NOT' description: 'tested obsoleting Object'! !

!SUnitExtensionsTest methodsFor: 'test' stamp: ''!
testShouldFix

	| testCase testResult  |
	
	testCase := self class selector: #shouldFixTest.
	testResult := testCase run.
	
	self assert: (testResult passed includes: testCase).
	self assert: testResult passed size=1.
	self assert: testResult failures isEmpty.
	self assert: testResult errors isEmpty.
	
	! !

!SUnitExtensionsTest methodsFor: 'test' stamp: 'mx 3/20/2006 23:40'!
testShouldRaiseWithExceptionDo

	| testCase testResult  |
	
	testCase := self class selector: #shouldRaiseWithExceptionDoTest.
	testResult := testCase run.
	
	self assert: (testResult passed includes: testCase).
	self assert: testResult passed size=1.
	self assert: testResult failures isEmpty.
	self assert: testResult errors isEmpty.
	
	! !

!SUnitExtensionsTest methodsFor: 'test' stamp: 'md 8/2/2006 11:06'!
testValidShouldNotTakeMoreThan
	| testCase testResult |

	testCase := self class selector: #validShouldNotTakeMoreThan.
	testResult := testCase run.

	self assert: (testResult passed includes: testCase).
	self assert: testResult passed size = 1.
	self assert: testResult failures isEmpty.
	self assert: testResult errors isEmpty

! !

!SUnitExtensionsTest methodsFor: 'test' stamp: ''!
testValidShouldNotTakeMoreThanMilliseconds

	| testCase testResult |

	testCase := self class selector: #validShouldNotTakeMoreThanMilliseconds.
	testResult := testCase run.

	self assert: (testResult passed includes: testCase).
	self assert: testResult passed size = 1.
	self assert: testResult failures isEmpty.
	self assert: testResult errors isEmpty

! !

!SUnitExtensionsTest methodsFor: 'real tests' stamp: 'md 8/2/2006 11:05'!
validShouldNotTakeMoreThan

	self should: [(Delay forMilliseconds: 100) wait] notTakeMoreThan:  200 milliSeconds.! !

!SUnitExtensionsTest methodsFor: 'real tests' stamp: ''!
validShouldNotTakeMoreThanMilliseconds

	self should: [(Delay forMilliseconds: 100) wait] notTakeMoreThanMilliseconds: 200! !

TestCase subclass: #SUnitTest
	instanceVariableNames: 'hasRun hasSetup hasRanOnce'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SUnit-Tests'!

!SUnitTest commentStamp: '<historical>' prior: 0!
This is both an example of writing tests and a self test for the SUnit. The tests 
here are pretty strange, since you want to make sure things blow up. You should 
not generally have to write tests this complicated in structure, although they 
will be far more complicated in terms of your own objects- more assertions, more 
complicated setup. Kent says: "Never forget, however, that if the tests are hard 
to write, something is probably wrong with the design".!

!SUnitTest methodsFor: 'private' stamp: ''!
assertForTestResult: aResult runCount: aRunCount passed: aPassedCount failed: aFailureCount errors: anErrorCount

	self
		assert: aResult runCount = aRunCount;
		assert: aResult passedCount = aPassedCount;
		assert: aResult failureCount = aFailureCount;
		assert: aResult errorCount = anErrorCount
			! !

!SUnitTest methodsFor: 'private' stamp: ''!
error
	3 zork
			! !

!SUnitTest methodsFor: 'testing' stamp: 'md 2/22/2006 14:17'!
errorShouldntRaise
	self 
		shouldnt: [self someMessageThatIsntUnderstood] 
		raise: Notification new
			! !

!SUnitTest methodsFor: 'private' stamp: ''!
fail
	self assert: false
			! !

!SUnitTest methodsFor: 'accessing' stamp: ''!
hasRun
	^hasRun
			! !

!SUnitTest methodsFor: 'accessing' stamp: ''!
hasSetup
	^hasSetup
			! !

!SUnitTest methodsFor: 'private' stamp: ''!
noop
			! !

!SUnitTest methodsFor: 'private' stamp: ''!
setRun
	hasRun := true
			! !

!SUnitTest methodsFor: 'running' stamp: ''!
setUp
	hasSetup := true
			! !

!SUnitTest methodsFor: 'testing' stamp: ''!
testAssert
	self assert: true.
	self deny: false
			! !

!SUnitTest methodsFor: 'testing' stamp: ''!
testDefects
	| result suite error failure |
	suite := TestSuite new.
	suite addTest: (error := self class selector: #error).
	suite addTest: (failure := self class selector: #fail).
	result := suite run.
	self assert: result defects asArray = (Array with: error with: failure).
	self
		assertForTestResult: result
		runCount: 2
		passed: 0
		failed: 1
		errors: 1
			! !

!SUnitTest methodsFor: 'testing' stamp: ''!
testDialectLocalizedException

	self
		should: [TestResult signalFailureWith: 'Foo']
		raise: TestResult failure.
	self
		should: [TestResult signalErrorWith: 'Foo']
		raise: TestResult error.

			! !

!SUnitTest methodsFor: 'testing' stamp: ''!
testError

	| case result |

	case := self class selector: #error.
	result := case run.
	self
		assertForTestResult: result
		runCount: 1
		passed: 0
		failed: 0
		errors: 1.

	case := self class selector: #errorShouldntRaise.
	result := case run.
	self 
		assertForTestResult: result
		runCount: 1
		passed: 0
		failed: 0
		errors: 1
			! !

!SUnitTest methodsFor: 'testing' stamp: ''!
testException

	self
		should: [self error: 'foo']
		raise: TestResult error
			! !

!SUnitTest methodsFor: 'testing' stamp: ''!
testFail

	| case result |

	case := self class selector: #fail.
	result := case run.

	self
		assertForTestResult: result
		runCount: 1
		passed: 0
		failed: 1
		errors: 0
			! !

!SUnitTest methodsFor: 'testing' stamp: ''!
testIsNotRerunOnDebug

	| case |

	case := self class selector: #testRanOnlyOnce.
	case run.
	case debug
			! !

!SUnitTest methodsFor: 'testing' stamp: ''!
testRan

	| case |

	case := self class selector: #setRun.
	case run.
	self assert: case hasSetup.
	self assert: case hasRun
			! !

!SUnitTest methodsFor: 'testing' stamp: ''!
testRanOnlyOnce

	self assert: hasRanOnce ~= true.
	hasRanOnce := true
			! !

!SUnitTest methodsFor: 'testing' stamp: ''!
testResult

	| case result |

	case := self class selector: #noop.
	result := case run.

	self
		assertForTestResult: result
		runCount: 1
		passed: 1
		failed: 0
		errors: 0
			! !

!SUnitTest methodsFor: 'testing' stamp: 'md 2/22/2006 14:16'!
testRunning

	(Delay forSeconds: 2) wait
			! !

!SUnitTest methodsFor: 'testing' stamp: 'kph 11/2/2006 00:32'!
testSelectorWithArg: anObject 
	"should not result in error, since it should not be run"
	^ self error! !

!SUnitTest methodsFor: 'testing' stamp: 'kph 11/2/2006 02:18'!
testSelectorsMatching
	| selectors |
	selectors := self class selectorsMatching: 'error*'.
	self assert: selectors includes: #error.
	self assert: selectors includes: #errorShouldntRaise.
	self assert: selectors size = 2.
	selectors := self class selectorsMatching: 'error@private'.
	self assert: selectors includes: #error.
	self assert: selectors size = 1.
	selectors := self class selectorsMatching: 'err*@pri*'.
	self assert: selectors includes: #error.
	self assert: selectors size = 1.
	selectors := self class selectorsMatching: '*@pri*'.
	self assert: selectors size = 4! !

!SUnitTest methodsFor: 'testing' stamp: 'kph 11/2/2006 01:17'!
testSelectorsMatchingMethodPrefix
	| selectors |
	selectors := self class selectorsMatching: 'test*'.
	selectors
		do: [:selector | 
			self
				assert: (selector beginsWith: 'test').
			self assert: selector last ~= $:]! !

!SUnitTest methodsFor: 'testing' stamp: 'kph 11/2/2006 02:17'!
testSelectorsMatchingOr
	| selectors |
	selectors := self class selectorsMatching: 'error@private|testAssert*'.
	self assert: selectors includes: #error.
	self assert: selectors includes: #testAssert.
	self assert: selectors size = 2.
	 ! !

!SUnitTest methodsFor: 'testing' stamp: 'kph 11/2/2006 02:19'!
testSelectorsMatchingOrNot
	| selectors |
	selectors := self class selectorsMatching: 'error*|^*@private'.
 	self assert: selectors includes: #errorShouldntRaise.
	self assert: selectors size = 1! !

!SUnitTest methodsFor: 'testing' stamp: ''!
testShould

	self
		should: [true];
		shouldnt: [false]
			! !

!SUnitTest methodsFor: 'testing' stamp: ''!
testSuite

	| suite result |

	suite := TestSuite new.
	suite 
		addTest: (self class selector: #noop);
		addTest: (self class selector: #fail);
		addTest: (self class selector: #error).

	result := suite run.

	self
		assertForTestResult: result
		runCount: 3
		passed: 1
		failed: 1
		errors: 1
			! !

!SUnitTest methodsFor: 'testing' stamp: 'kph 12/15/2006 00:13'!
testSuiteSpecAsString

	| suite result |

	suite := self 
	suite 
		addTest: (self class selector: #noop);
		addTest: (self class selector: #fail);
		addTest: (self class selector: #error).

	result := suite run.

	self
		assertForTestResult: result
		runCount: 3
		passed: 1
		failed: 1
		errors: 1
			! !

!SUnitTest methodsFor: 'testing' stamp: 'DF 3/17/2006 01:30'!
testWithExceptionDo

	self
		should: [self error: 'foo']
		raise: TestResult error
		withExceptionDo: [:exception |
			self assert: (exception description includesSubString: 'foo')
		]
			! !

TestCase subclass: #SimpleTestResourceTestCase
	instanceVariableNames: 'resource'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SUnit-Tests'!

!SimpleTestResourceTestCase class methodsFor: 'Not categorized' stamp: ''!
resources
	^Set new add: SimpleTestResource; yourself
			! !

!SimpleTestResourceTestCase methodsFor: 'Not categorized' stamp: ''!
dummy
	self assert: true
			! !

!SimpleTestResourceTestCase methodsFor: 'Not categorized' stamp: ''!
error
	'foo' odd
			! !

!SimpleTestResourceTestCase methodsFor: 'Not categorized' stamp: ''!
fail
	self assert: false
			! !

!SimpleTestResourceTestCase methodsFor: 'Not categorized' stamp: ''!
setRun
	resource setRun
			! !

!SimpleTestResourceTestCase methodsFor: 'Not categorized' stamp: ''!
setUp
	resource := SimpleTestResource current
			! !

!SimpleTestResourceTestCase methodsFor: 'Not categorized' stamp: ''!
testRan
	| case |

	case := self class selector: #setRun.
	case run.
	self assert: resource hasSetup.
	self assert: resource hasRun
			! !

!SimpleTestResourceTestCase methodsFor: 'Not categorized' stamp: ''!
testResourceInitRelease
	| result suite error failure |
	suite := TestSuite new.
	suite addTest: (error := self class selector: #error).
	suite addTest: (failure := self class selector: #fail).
	suite addTest: (self class selector: #dummy).
	result := suite run.
	self assert: resource hasSetup
			! !

!SimpleTestResourceTestCase methodsFor: 'Not categorized' stamp: ''!
testResourcesCollection
	| collection |
	collection := self resources.
	self assert: collection size = 1
			! !

!TestCase class methodsFor: 'instance creation' stamp: ''!
debug: aSymbol

	^(self selector: aSymbol) debug
			! !

!TestCase class methodsFor: 'building suites' stamp: 'kph 10/26/2006 11:53'!
emptySuite
	^ TestSuite new! !

!TestCase class methodsFor: 'testing' stamp: 'md 2/22/2006 14:21'!
isAbstract
	"Override to true if a TestCase subclass is Abstract and should not have
	TestCase instances built from it"

	^self name = #TestCase
			! !

!TestCase class methodsFor: 'published suites' stamp: 'kph 10/30/2006 01:31'!
longTests
	^   'longtest*'! !

!TestCase class methodsFor: 'published suites' stamp: 'kph 12/14/2006 03:46'!
publishedSuites
	^ #(#standardTests "#longTests #standardAndLongTests")! !

!TestCase class methodsFor: 'accessing' stamp: ''!
resources

	^#()
			! !

!TestCase class methodsFor: 'instance creation' stamp: ''!
run: aSymbol

	^(self selector: aSymbol) run
			! !

!TestCase class methodsFor: 'instance creation' stamp: ''!
selector: aSymbol

	^self new setTestSelector: aSymbol
			! !

!TestCase class methodsFor: 'building suites' stamp: 'kph 12/14/2006 21:41'!
selectorsMatching: selectorMatch 

	| categoryAndSelector matches exclusions |

	matches := selectorMatch splitOn: '|'.
	exclusions := matches select: [:match | match first = $^].
	matches removeAll: exclusions.
	exclusions := exclusions collect: [:match | match copyWithoutFirst].
	
	^ (self shouldInheritSelectors
		ifTrue: [self allSelectors]
		ifFalse: [self selectors])
		select: [:selector | "unary selectors only"
			(selector last ~= $:
					and: ["match against selector@category"
						categoryAndSelector := selector , '@' , (self organization categoryOfElement: selector) asString.
						matches
							anySatisfy: [:match | match match: categoryAndSelector]])
				and: [exclusions
						noneSatisfy: [:exclusion | exclusion match: categoryAndSelector]]]! !

!TestCase class methodsFor: 'testing' stamp: 'kph 11/2/2006 02:14'!
shouldInheritSelectors
	"I should inherit from an Abstract superclass but not from a concrete
	one by default, If a test case with selectors wants to inherit selectors 
	from a concrete superclass, override this to true in that subclass."
	^ self superclass isAbstract
		and: [self superclass name ~= #TestCase]! !

!TestCase class methodsFor: 'published suites' stamp: 'kph 12/14/2006 03:47'!
standardTests
	^  'test*'! !

!TestCase class methodsFor: 'building suites' stamp: 'kph 12/14/2006 17:01'!
suite
	^ self suite: #standardTests! !

!TestCase class methodsFor: 'building suites' stamp: 'kph 12/14/2006 21:42'!
suite: publishedSuiteSelectorOrMatchString
	^ self suite: publishedSuiteSelectorOrMatchString addTo: self emptySuite! !

!TestCase class methodsFor: 'building suites' stamp: 'kph 12/15/2006 00:48'!
suite: spec addTo: suite 

	"spec specifies a test suite as either 
		a) a selector (symbol) to be performed if possible to obtain either b or c.
		b) an explicit match string
		c) an array of items over which to iterate."
		
	| theSpec classes |
	
	theSpec := spec isSymbol 
		ifTrue: [ (self respondsTo: spec) ifTrue: [self perform: spec ] ifFalse: [ nil ] ] 
		ifFalse: [ spec ].
		
	theSpec isArray
		ifTrue: [ 
			theSpec do: [ :eachSpec | self suite: eachSpec addTo: suite ].
			^suite ].
		
	theSpec ifNil: [^ suite].
	
	classes := self isAbstract ifTrue: [ self allSubclasses reject: [:c | c isAbstract] ] ifFalse: [ Array with: self ].
	
	classes do: [ :testClass |
		(testClass selectorsMatching: theSpec) asSortedCollection
			do: [:each | suite addTest: (testClass selector: each)].
	].

	^ suite! !

!TestCase class methodsFor: 'accessing' stamp: 'kph 10/26/2006 05:56'!
sunitVersion
	^ '3.2b'! !

!TestCase methodsFor: 'dependencies' stamp: ''!
addDependentToHierachy: anObject 
	"an empty method. for Composite compability with TestSuite"


			! !

!TestCase methodsFor: 'accessing' stamp: 'md 8/2/2006 10:59'!
assert: aBooleanOrBlock

	aBooleanOrBlock value ifFalse: [self signalFailure: 'Assertion failed']
			! !

!TestCase methodsFor: 'accessing' stamp: 'md 8/2/2006 11:00'!
assert: aBooleanOrBlock description: aString
	aBooleanOrBlock value ifFalse: [
		self logFailure: aString.
		TestResult failure signal: aString]
			! !

!TestCase methodsFor: 'accessing' stamp: 'md 8/2/2006 11:00'!
assert: aBooleanOrBlock description: aString resumable: resumableBoolean 
	| exception |
	aBooleanOrBlock value
		ifFalse: 
			[self logFailure: aString.
			exception := resumableBoolean
						ifTrue: [TestResult resumableFailure]
						ifFalse: [TestResult failure].
			exception signal: aString]
			! !

!TestCase methodsFor: 'accessing' stamp: 'kph 11/2/2006 01:30'!
assert: a equals: b 
	^ self assert: (a = b)
! !

!TestCase methodsFor: 'accessing' stamp: 'kph 11/2/2006 01:29'!
assert: a includes: b 
	^ self assert: (a includes: b)
! !

!TestCase methodsFor: 'running' stamp: 'md 2/22/2006 14:27'!
debug
	self resources do: [:res | 
		res isAvailable ifFalse: [^res signalInitializationError]].
	[(self class selector: testSelector) runCase] 
		ensure: [self resources do: [:each | each reset]]
			! !

!TestCase methodsFor: 'running' stamp: 'bp 11/15/2004 18:13'!
debugAsFailure
	| semaphore |
	semaphore := Semaphore new.
	self resources do: [:res | 
		res isAvailable ifFalse: [^res signalInitializationError]].
	[semaphore wait. self resources do: [:each | each reset]] fork.
	(self class selector: testSelector) runCaseAsFailure: semaphore.! !

!TestCase methodsFor: 'accessing' stamp: 'md 8/2/2006 11:00'!
deny: aBooleanOrBlock

	self assert: aBooleanOrBlock value not
			! !

!TestCase methodsFor: 'accessing' stamp: 'md 8/2/2006 11:00'!
deny: aBooleanOrBlock description: aString
	self assert: aBooleanOrBlock value not description: aString
			! !

!TestCase methodsFor: 'accessing' stamp: 'md 8/2/2006 11:00'!
deny: aBooleanOrBlock description: aString resumable: resumableBoolean 
	self
		assert: aBooleanOrBlock value not
		description: aString
		resumable: resumableBoolean
			! !

!TestCase methodsFor: 'private' stamp: 'md 2/22/2006 14:27'!
executeShould: aBlock inScopeOf: anExceptionalEvent 
	^[aBlock value.
 	false] on: anExceptionalEvent
		do: [:ex | ex return: true]
			! !

!TestCase methodsFor: 'private' stamp: 'md 2/22/2006 14:27'!
executeShould: aBlock inScopeOf: anExceptionalEvent withDescriptionContaining: aString
	^[aBlock value.
 	false] on: anExceptionalEvent
		do: [:ex | ex return: (ex description includesSubString: aString) ]
			! !

!TestCase methodsFor: 'private' stamp: 'md 2/22/2006 14:27'!
executeShould: aBlock inScopeOf: anExceptionalEvent withDescriptionNotContaining: aString
	^[aBlock value.
 	false] on: anExceptionalEvent
		do: [:ex | ex return: (ex description includesSubString: aString) not ]
			! !

!TestCase methodsFor: 'extensions' stamp: 'mx 3/20/2006 23:32'!
executeShould: aBlock inScopeOf: anException withExceptionDo: anotherBlock

	^[aBlock value.
 	false] 
		on: anException
		do: [:exception | 
			anotherBlock value: exception.
			exception return: true]! !

!TestCase methodsFor: 'testing' stamp: 'JF 7/30/2003 13:40'!
expectedFailures
	^ Array new! !

!TestCase methodsFor: 'extensions' stamp: 'mx 3/13/2006 23:21'!
fail

	^self assert: false! !

!TestCase methodsFor: 'running' stamp: 'md 2/22/2006 14:17'!
failureLog	
	^Transcript

			! !

!TestCase methodsFor: 'running' stamp: ''!
isLogging
	"By default, we're not logging failures. If you override this in 
	a subclass, make sure that you override #failureLog"
	^false
			! !

!TestCase methodsFor: 'running' stamp: ''!
logFailure: aString
	self isLogging ifTrue: [
		self failureLog 
			cr; 
			nextPutAll: aString; 
			flush]
			! !

!TestCase methodsFor: 'running' stamp: 'bp 11/15/2004 18:17'!
openDebuggerOnFailingTestMethod
	"SUnit has halted one step in front of the failing test method. Step over the 'self halt' and 
	 send into 'self perform: testSelector' to see the failure from the beginning"

	self
		halt;
		performTest! !

!TestCase methodsFor: 'private' stamp: 'md 2/22/2006 14:22'!
performTest

	self perform: testSelector asSymbol
			! !

!TestCase methodsFor: 'printing' stamp: ''!
printOn: aStream

	aStream
		nextPutAll: self class printString;
		nextPutAll: '>>#';
		nextPutAll: testSelector
			! !

!TestCase methodsFor: 'dependencies' stamp: ''!
removeDependentFromHierachy: anObject 
	"an empty method. for Composite compability with TestSuite"


			! !

!TestCase methodsFor: 'accessing' stamp: ''!
resources
	| allResources resourceQueue |
	allResources := Set new.
	resourceQueue := OrderedCollection new.
	resourceQueue addAll: self class resources.
	[resourceQueue isEmpty] whileFalse: [
		| next |
		next := resourceQueue removeFirst.
		allResources add: next.
		resourceQueue addAll: next resources].
	^allResources
			! !

!TestCase methodsFor: 'running' stamp: 'kph 11/1/2006 20:21'!
run
	| result |
	result := TestResult new.
	self runOn: result.
	^ result! !

!TestCase methodsFor: 'running' stamp: 'md 2/22/2006 14:27'!
runCase

	[self setUp.
	self performTest] ensure: [self tearDown]
			! !

!TestCase methodsFor: 'running' stamp: 'md 2/22/2006 14:27'!
runCaseAsFailure: aSemaphore
	[self setUp.
	self openDebuggerOnFailingTestMethod] ensure: [
		self tearDown.
		aSemaphore signal]! !

!TestCase methodsFor: 'running' stamp: 'kph 11/1/2006 20:20'!
runOn: aResult 
	aResult runCase: self! !

!TestCase methodsFor: 'accessing' stamp: ''!
selector
	^testSelector
			! !

!TestCase methodsFor: 'private' stamp: ''!
setTestSelector: aSymbol
	testSelector := aSymbol
			! !

!TestCase methodsFor: 'running' stamp: ''!
setUp
			! !

!TestCase methodsFor: 'accessing' stamp: ''!
should: aBlock
	self assert: aBlock value
			! !

!TestCase methodsFor: 'accessing' stamp: ''!
should: aBlock description: aString
	self assert: aBlock value description: aString
			! !

!TestCase methodsFor: 'extensions' stamp: 'md 8/2/2006 11:09'!
should: aBlock notTakeMoreThan: aDuration
    "Evaluate aBlock in a forked process and if it takes more than anInteger milliseconds
    to run we terminate the process and report a test failure.  It'' important to
    use the active process for the test failure so that the failure reporting works correctly
    in the context of the exception handlers."

    | evaluated evaluationProcess result delay testProcess |

    evaluated := false.
    delay := Delay forDuration: aDuration.
    testProcess := Processor activeProcess.
    "Create a new process to evaluate aBlock"
    evaluationProcess := [
        result := aBlock value.
        evaluated := true.
        delay unschedule.
        testProcess resume ] forkNamed: 'Process to evaluate should: notTakeMoreThanMilliseconds:'.

    "Wait the milliseconds they asked me to"
    delay wait.
    "After this point either aBlock was evaluated or not..."
    evaluated ifFalse: [
        evaluationProcess terminate.
        self assert: false description: ('Block evaluation took more than the expected <1p>' expandMacrosWith: aDuration)].
   
    ^result! !

!TestCase methodsFor: 'extensions' stamp: 'md 8/2/2006 11:08'!
should: aBlock notTakeMoreThanMilliseconds: anInteger
    "For compatibility with other Smalltalks"

   self should: aBlock notTakeMoreThan: (Duration milliSeconds: anInteger).! !

!TestCase methodsFor: 'accessing' stamp: ''!
should: aBlock raise: anExceptionalEvent 
	^self assert: (self executeShould: aBlock inScopeOf: anExceptionalEvent)
			! !

!TestCase methodsFor: 'accessing' stamp: ''!
should: aBlock raise: anExceptionalEvent description: aString 
	^self assert: (self executeShould: aBlock inScopeOf: anExceptionalEvent)
		description: aString
			! !

!TestCase methodsFor: 'accessing' stamp: 'nk 5/11/2003 10:32'!
should: aBlock raise: anExceptionalEvent whoseDescriptionDoesNotInclude: subString description: aString 
	^self assert: (self executeShould: aBlock inScopeOf: anExceptionalEvent withDescriptionNotContaining: subString)
		description: aString
! !

!TestCase methodsFor: 'accessing' stamp: 'nk 5/11/2003 10:24'!
should: aBlock raise: anExceptionalEvent whoseDescriptionIncludes: subString description: aString 
	^self assert: (self executeShould: aBlock inScopeOf: anExceptionalEvent withDescriptionContaining: subString)
		description: aString
! !

!TestCase methodsFor: 'extensions' stamp: 'mx 3/20/2006 23:52'!
should: aBlock raise: anException withExceptionDo: anotherBlock 

	^self assert: (self executeShould: aBlock inScopeOf: anException withExceptionDo: anotherBlock)! !

!TestCase methodsFor: 'extensions' stamp: 'mx 3/20/2006 21:29'!
shouldFix: aBlock 

	^self should: aBlock raise: Exception! !

!TestCase methodsFor: 'testing' stamp: 'JF 7/30/2003 13:39'!
shouldPass
	"Unless the selector is in the list we get from #expectedFailures, we expect it to pass"
	^ (self expectedFailures includes: testSelector) not! !

!TestCase methodsFor: 'accessing' stamp: ''!
shouldnt: aBlock
	self deny: aBlock value
			! !

!TestCase methodsFor: 'accessing' stamp: ''!
shouldnt: aBlock description: aString
	self deny: aBlock value description: aString
			! !

!TestCase methodsFor: 'accessing' stamp: ''!
shouldnt: aBlock raise: anExceptionalEvent 
	^self assert: (self executeShould: aBlock inScopeOf: anExceptionalEvent) not
			! !

!TestCase methodsFor: 'accessing' stamp: ''!
shouldnt: aBlock raise: anExceptionalEvent description: aString 
	^self assert: (self executeShould: aBlock inScopeOf: anExceptionalEvent) not 		description: aString
			! !

!TestCase methodsFor: 'accessing' stamp: 'nk 5/11/2003 10:34'!
shouldnt: aBlock raise: anExceptionalEvent whoseDescriptionDoesNotInclude: subString description: aString 
	^self assert: (self executeShould: aBlock inScopeOf: anExceptionalEvent withDescriptionNotContaining: subString) not
		description: aString
! !

!TestCase methodsFor: 'accessing' stamp: 'nk 5/11/2003 10:34'!
shouldnt: aBlock raise: anExceptionalEvent whoseDescriptionIncludes: subString description: aString 
	^self assert: (self executeShould: aBlock inScopeOf: anExceptionalEvent withDescriptionContaining: subString) not
		description: aString
! !

!TestCase methodsFor: 'accessing' stamp: 'md 2/22/2006 14:26'!
signalFailure: aString
	TestResult failure signal: aString! !

!TestCase methodsFor: 'running' stamp: ''!
tearDown
			! !

Object subclass: #TestResource
	instanceVariableNames: 'name description'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SUnit-Kernel'!

TestResource class
	instanceVariableNames: 'current'!

TestResource class
	instanceVariableNames: 'current'!

TestResource subclass: #ClassClonerTestResource
	instanceVariableNames: 'clones'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SUnit-Extensions'!

!ClassClonerTestResource commentStamp: 'kph 12/20/2006 20:01' prior: 0!
To enable testing of class-side behaviour without disturbing the class, or its superclasses which are singletons in use.

This TestResource provides a clone of an existing class on demand. Clones are created to behave in the same manner as the original class except that class vars are local copies, and its methods have been recompiled to reference them.

usage: (in fixture setup)

"obtain the clone"
clonedClass := ClassClonerTestResource current cloneOf: DateAndTime.
"the clone needs to be explicitly reset to a known state as needed before each test."
clonedClass initialize.!

!ClassClonerTestResource methodsFor: 'as yet unclassified' stamp: 'kph 12/20/2006 19:49'!
cloneOf: origClass

	^ clones at: origClass ifAbsentPut: [ self makeCloneOf: origClass ]! !

!ClassClonerTestResource methodsFor: 'as yet unclassified' stamp: 'kph 12/20/2006 19:47'!
makeCloneOf: origClass

	| clone |

	"copy the class"
	clone := origClass copy.

	origClass classPool keysAndValuesDo: [ :k :v | 
		 clone classPool at: k deepCopy put: v deepCopy.
	].
 
 	clone compileAll.
 
	^clone! !

!ClassClonerTestResource methodsFor: 'as yet unclassified' stamp: 'kph 12/20/2006 19:47'!
setUp

clones := IdentityDictionary new.! !

!ClassClonerTestResource methodsFor: 'as yet unclassified' stamp: 'kph 12/20/2006 19:47'!
tearDown

clones := nil.! !

TestResource subclass: #SimpleTestResource
	instanceVariableNames: 'runningState hasRun hasSetup hasRanOnce'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SUnit-Tests'!

!SimpleTestResource methodsFor: 'testing' stamp: ''!
hasRun
	^hasRun
			! !

!SimpleTestResource methodsFor: 'testing' stamp: ''!
hasSetup
	^hasSetup
			! !

!SimpleTestResource methodsFor: 'testing' stamp: ''!
isAvailable
	
	^self runningState == self startedStateSymbol
			! !

!SimpleTestResource methodsFor: 'accessing' stamp: ''!
runningState

	^runningState
			! !

!SimpleTestResource methodsFor: 'accessing' stamp: ''!
runningState: aSymbol

	runningState := aSymbol
			! !

!SimpleTestResource methodsFor: 'running' stamp: ''!
setRun
	hasRun := true
			! !

!SimpleTestResource methodsFor: 'running' stamp: ''!
setUp
	
	self runningState: self startedStateSymbol.
	hasSetup := true
			! !

!SimpleTestResource methodsFor: 'running' stamp: ''!
startedStateSymbol

	^#started
			! !

!SimpleTestResource methodsFor: 'running' stamp: ''!
stoppedStateSymbol

	^#stopped
			! !

!SimpleTestResource methodsFor: 'running' stamp: ''!
tearDown

	self runningState: self stoppedStateSymbol
			! !

!TestResource class methodsFor: 'accessing' stamp: ''!
current

	current isNil
		ifTrue: [current := self new].

	^current
			! !

!TestResource class methodsFor: 'accessing' stamp: ''!
current: aTestResource

	current := aTestResource
			! !

!TestResource class methodsFor: 'testing' stamp: 'md 2/22/2006 14:21'!
isAbstract
	"Override to true if a TestResource subclass is Abstract and should not have
	TestCase instances built from it"

	^self name = #TestResource
			! !

!TestResource class methodsFor: 'testing' stamp: ''!
isAvailable
	^self current notNil and: [self current isAvailable]
			! !

!TestResource class methodsFor: 'testing' stamp: ''!
isUnavailable

	^self isAvailable not
			! !

!TestResource class methodsFor: 'creation' stamp: ''!
reset

	current notNil ifTrue: [
		[current tearDown] ensure: [
			current := nil]]
			! !

!TestResource class methodsFor: 'accessing' stamp: ''!
resources
	^#()
			! !

!TestResource class methodsFor: 'creation' stamp: ''!
signalInitializationError
	^TestResult signalErrorWith: 'Resource ' , self name , ' could not be initialized'
			! !

!TestResource methodsFor: 'accessing' stamp: ''!
description

	description isNil
		ifTrue: [^''].

	^description
			! !

!TestResource methodsFor: 'accessing' stamp: ''!
description: aString

	description := aString
			! !

!TestResource methodsFor: 'initializing' stamp: ''!
initialize
	self setUp

			! !

!TestResource methodsFor: 'testing' stamp: ''!
isAvailable
	"override to provide information on the
	readiness of the resource"
	
	^true
			! !

!TestResource methodsFor: 'testing' stamp: ''!
isUnavailable
	"override to provide information on the
	readiness of the resource"
	
	^self isAvailable not
			! !

!TestResource methodsFor: 'accessing' stamp: ''!
name

	name isNil
		ifTrue: [^self printString].

	^name
			! !

!TestResource methodsFor: 'accessing' stamp: ''!
name: aString

	name := aString
			! !

!TestResource methodsFor: 'printing' stamp: ''!
printOn: aStream

	aStream nextPutAll: self class printString
			! !

!TestResource methodsFor: 'accessing' stamp: ''!
resources
	^self class resources
			! !

!TestResource methodsFor: 'running' stamp: ''!
setUp
	"Does nothing. Subclasses should override this
	to initialize their resource"
			! !

!TestResource methodsFor: 'running' stamp: ''!
signalInitializationError
	^self class signalInitializationError
			! !

!TestResource methodsFor: 'running' stamp: ''!
tearDown
	"Does nothing. Subclasses should override this
	to tear down their resource"
			! !

Object subclass: #TestResult
	instanceVariableNames: 'failures errors passed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SUnit-Kernel'!

!TestResult commentStamp: '<historical>' prior: 0!
This is a Collecting Parameter for the running of a bunch of tests. TestResult is an interesting object to subclass or substitute. #runCase: is the external protocol you need to reproduce. Kent has seen TestResults that recorded coverage information and that sent email when they were done.!

!TestResult class methodsFor: 'exceptions' stamp: ''!
error
	^self exError
			! !

!TestResult class methodsFor: 'exceptions' stamp: 'md 2/22/2006 14:17'!
exError
	^Error
			! !

!TestResult class methodsFor: 'exceptions' stamp: ''!
failure
	^TestFailure
			! !

!TestResult class methodsFor: 'exceptions' stamp: ''!
resumableFailure
	^ResumableTestFailure
			! !

!TestResult class methodsFor: 'exceptions' stamp: 'md 2/22/2006 14:26'!
signalErrorWith: aString 
	self error signal: aString
			! !

!TestResult class methodsFor: 'exceptions' stamp: 'md 2/22/2006 14:26'!
signalFailureWith: aString 
	self failure signal: aString
			! !

!TestResult methodsFor: 'accessing' stamp: ''!
correctCount
	"depreciated - use #passedCount"

	^self passedCount
			! !

!TestResult methodsFor: 'accessing' stamp: ''!
defects
	^OrderedCollection new
		addAll: self errors;
		addAll: self failures; yourself
			! !

!TestResult methodsFor: 'accessing' stamp: ''!
errorCount

	^self errors size
			! !

!TestResult methodsFor: 'compatibility' stamp: 'JF 7/30/2003 16:09'!
errors
	^ self unexpectedErrors! !

!TestResult methodsFor: 'accessing' stamp: 'JF 7/30/2003 13:54'!
expectedDefectCount
	^ self expectedDefects size! !

!TestResult methodsFor: 'accessing' stamp: 'md 11/25/2004 16:36'!
expectedDefects
	^ (errors, failures asOrderedCollection) select: [:each | each shouldPass not] ! !

!TestResult methodsFor: 'accessing' stamp: 'JF 7/30/2003 13:54'!
expectedPassCount
	^ self expectedPasses size! !

!TestResult methodsFor: 'accessing' stamp: 'JF 7/30/2003 16:14'!
expectedPasses
	^ passed select: [:each | each shouldPass] ! !

!TestResult methodsFor: 'accessing' stamp: 'kph 11/1/2006 20:42'!
fail: a dueTo: b! !

!TestResult methodsFor: 'accessing' stamp: ''!
failureCount

	^self failures size
			! !

!TestResult methodsFor: 'compatibility' stamp: 'md 11/25/2004 16:23'!
failures
	^ self unexpectedFailures, self unexpectedPasses ! !

!TestResult methodsFor: 'testing' stamp: ''!
hasErrors

	^self errors size > 0
			! !

!TestResult methodsFor: 'testing' stamp: ''!
hasFailures

	^self failures size > 0
			! !

!TestResult methodsFor: 'testing' stamp: 'JF 7/30/2003 14:04'!
hasPassed
	^self runCount = (self passedCount + self expectedDefectCount)! !

!TestResult methodsFor: 'initialize-release' stamp: 'stephaneducasse 2/3/2006 22:41'!
initialize
	passed := OrderedCollection new.
	failures := Set new.
	errors := OrderedCollection new.! !

!TestResult methodsFor: 'testing' stamp: ''!
isError: aTestCase

	^self errors includes: aTestCase
			! !

!TestResult methodsFor: 'testing' stamp: ''!
isFailure: aTestCase
	^self failures includes: aTestCase
			! !

!TestResult methodsFor: 'testing' stamp: ''!
isPassed: aTestCase

	^self passed includes: aTestCase
			! !

!TestResult methodsFor: 'accessing' stamp: 'kph 11/1/2006 20:41'!
pass: a! !

!TestResult methodsFor: 'compatibility' stamp: 'JF 7/30/2003 16:08'!
passed
	^ self expectedPasses, self expectedDefects! !

!TestResult methodsFor: 'accessing' stamp: ''!
passedCount

	^self passed size
			! !

!TestResult methodsFor: 'printing' stamp: 'JF 7/30/2003 16:15'!
printOn: aStream
	aStream
		nextPutAll: self runCount printString;
		nextPutAll: ' run, ';
		nextPutAll: self expectedPassCount printString;
		nextPutAll: ' passes, ';
		nextPutAll: self expectedDefectCount printString;
		nextPutAll:' expected failures, ';
		nextPutAll: self unexpectedFailureCount printString;
		nextPutAll: ' failures, ';
		nextPutAll: self unexpectedErrorCount printString;
		nextPutAll:' errors, ';
		nextPutAll: self unexpectedPassCount printString;
		nextPutAll:' unexpected passes'.! !

!TestResult methodsFor: 'running' stamp: 'md 2/22/2006 14:27'!
runCase: aTestCase
	| testCasePassed |
	testCasePassed := true.
	[[aTestCase runCase] 
			on: self class failure
			do: 
				[:signal | 
				failures add: aTestCase.
				testCasePassed := false.
				signal return: false]]
					on: self class error
					do:
						[:signal |
						errors add: aTestCase.
						testCasePassed := false.
						signal return: false].
	testCasePassed ifTrue: [passed add: aTestCase]! !

!TestResult methodsFor: 'accessing' stamp: 'JF 7/30/2003 16:07'!
runCount
	^ passed size + failures size + errors size! !

!TestResult methodsFor: 'accessing' stamp: 'JF 7/30/2003 16:06'!
tests
	^(OrderedCollection new: self runCount)
		addAll: passed;
		addAll: failures;
		addAll: errors;
		yourself! !

!TestResult methodsFor: 'accessing' stamp: 'JF 7/30/2003 13:54'!
unexpectedErrorCount
	^ self unexpectedErrors size! !

!TestResult methodsFor: 'accessing' stamp: 'JF 7/30/2003 16:14'!
unexpectedErrors
	^ errors select: [:each | each shouldPass] ! !

!TestResult methodsFor: 'accessing' stamp: 'JF 7/30/2003 13:54'!
unexpectedFailureCount
	^ self unexpectedFailures size! !

!TestResult methodsFor: 'accessing' stamp: 'JF 7/30/2003 16:14'!
unexpectedFailures
	^ failures select: [:each | each shouldPass] ! !

!TestResult methodsFor: 'accessing' stamp: 'JF 7/30/2003 13:54'!
unexpectedPassCount
	^ self unexpectedPasses size! !

!TestResult methodsFor: 'accessing' stamp: 'JF 7/30/2003 16:14'!
unexpectedPasses
	^ passed select: [:each | each shouldPass not] ! !

Object subclass: #TestSuite
	instanceVariableNames: 'tests resources name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SUnit-Kernel'!

!TestSuite commentStamp: '<historical>' prior: 0!
This is a Composite of Tests, either TestCases or other TestSuites. The common protocol is #run: aTestResult and the dependencies protocol!

!TestSuite class methodsFor: 'instance creation' stamp: ''!
named: aString

	^self new
		name: aString;
		yourself
			! !

!TestSuite methodsFor: 'accessing' stamp: 'kph 11/1/2006 20:24'!
add: aTest 
	self tests add: aTest! !

!TestSuite methodsFor: 'dependencies' stamp: 'md 2/22/2006 14:24'!
addDependentToHierachy: anObject
	self addDependent: anObject.
	self tests do: [ :each | each addDependentToHierachy: anObject]
			! !

!TestSuite methodsFor: 'accessing' stamp: ''!
addTest: aTest
	self tests add: aTest
			! !

!TestSuite methodsFor: 'accessing' stamp: ''!
addTests: aCollection 
	aCollection do: [:eachTest | self addTest: eachTest]
			! !

!TestSuite methodsFor: 'accessing' stamp: ''!
defaultResources
	^self tests 
		inject: Set new
		into: [:coll :testCase | 
			coll
				addAll: testCase resources;
				yourself]
			! !

!TestSuite methodsFor: 'accessing' stamp: ''!
name

	^name
			! !

!TestSuite methodsFor: 'accessing' stamp: ''!
name: aString

	name := aString
			! !

!TestSuite methodsFor: 'dependencies' stamp: 'md 2/22/2006 14:24'!
removeDependentFromHierachy: anObject
	self removeDependent: anObject.
	self tests do: [ :each | each removeDependentFromHierachy: anObject]
			! !

!TestSuite methodsFor: 'accessing' stamp: ''!
resources
	resources isNil ifTrue: [resources := self defaultResources].
	^resources
			! !

!TestSuite methodsFor: 'accessing' stamp: ''!
resources: anObject
	resources := anObject
			! !

!TestSuite methodsFor: 'running' stamp: 'kph 11/1/2006 20:21'!
run
	| result |
	result := TestResult new.
	self resources
		do: [:res | res isAvailable
				ifFalse: [^ res signalInitializationError]].
	[self runOn: result]
		ensure: [self resources
				do: [:each | each reset]].
	^ result! !

!TestSuite methodsFor: 'running' stamp: 'kph 11/1/2006 20:21'!
runOn: aResult 
	self tests
		do: [:each | 
			self changed: each.
			each runOn: aResult]! !

!TestSuite methodsFor: 'accessing' stamp: ''!
tests
	tests isNil ifTrue: [tests := OrderedCollection new].
	^tests
			! !

Exception subclass: #TestFailure
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SUnit-Kernel'!

!TestFailure commentStamp: '<historical>' prior: 0!
Signaled in case of a failed test (failure). The test framework distinguishes between failures and errors. A failure is anticipated and checked for with assertions. Errors are unanticipated problems like a division by 0 or an index out of bounds ...!

TestFailure subclass: #ResumableTestFailure
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SUnit-Kernel'!

!ResumableTestFailure commentStamp: '<historical>' prior: 0!
A ResumableTestFailure triggers a TestFailure, but lets execution of the TestCase continue. this is useful when iterating through collections, and #assert: ing on each element. in combination with methods like testcase>>#assert:description:, this lets you run through a whole collection and note which tests pass.

here''s an example:

	

	(1 to: 30) do: [ :each |
		self assert: each odd description: each printString, ' is even' resumable: true]

for each element where #odd returns <false>, the element will be printed to the Transcript. !

!ResumableTestFailure methodsFor: 'camp smalltalk' stamp: ''!
isResumable
	"Of course a ResumableTestFailure is resumable ;-)"

	^true! !

!ResumableTestFailure methodsFor: 'camp smalltalk' stamp: ''!
sunitExitWith: aValue
	self resume: aValue! !

!TestFailure methodsFor: 'camp smalltalk' stamp: 'ajh 1/24/2003 19:23'!
defaultAction

	Processor activeProcess
		debug: self signalerContext
		title: self description! !

!TestFailure methodsFor: 'camp smalltalk' stamp: 'ajh 2/1/2003 00:58'!
isResumable
	
	^ false! !
ClassClonerTestResourceTest initialize!
